/*! Tripetto Collector 1.7.1-26 - Copyright (C) 2019 Tripetto B.V. - All Rights Reserved */

declare module "@marblecore/functional" {
    /** Implicit exports */
    export * from "@marblecore/functional/lib/functional";
    /** Modules */
    export { Console } from "@marblecore/functional/lib/console";
    export { Environment } from "@marblecore/functional/lib/environment";
    export { Num } from "@marblecore/functional/lib/number";
    export { Str } from "@marblecore/functional/lib/string";
    /** Explicit exports */
    export {
        TAny,
        TArguments,
        TList,
        ICollection,
        IEnumerable,
        IObject
    } from "@marblecore/functional/lib/types";
    export { DateTime } from "@marblecore/functional/lib/date";
    export { Flags } from "@marblecore/functional/lib/flags";
    export { Enumerator } from "@marblecore/functional/lib/enumerator";
    export {
        Limiter,
        Throttle,
        Debounce,
        TType as TLimiterType
    } from "@marblecore/functional/lib/limiter";
    export { Stack } from "@marblecore/functional/lib/stack";
    /** Indicates if debugging is enabled. */
    export let DEBUG: boolean;
    /** Enable debugging. */
    export function enableDebugging(): void;
    /** Disable debugging. */
    export function disableDebugging(): void;
}

declare module "@marblecore/functional/lib/functional" {
    import {
        IObject,
        TAny,
        TArguments,
        TList
    } from "@marblecore/functional/lib/types";
    /** The mother of all helper functions. Does nothing. */
    export function noop(): void;
    /**
     * Validates if the supplied variable is undefined.
     *
     * #### Example
     * ```typescript
     * isUndefined(1); // Returns `false`
     * isUndefined(); // Returns `true`
     * ```
     *
     * @param pVariable Variable to validate.
     * @return Returns `true` if the variable is undefined.
     */
    export function isUndefined(pVariable: TAny): pVariable is undefined;
    /**
     * Validates if the supplied variable is defined.
     *
     * #### Example
     * ```typescript
     * isDefined(1); // Returns `true`
     * isDefined(); // Returns `false`
     * ```
     *
     * @param pVariable Variable to validate.
     * @return Returns `true` if the variable is defined.
     */
    export function isDefined(pVariable: TAny): boolean;
    /**
     * Validates if the supplied variable is `null`. Please consider the use of
     * `null`. TypeScript has two bottom types: `null` and `undefined`. They are
     * intented to mean different things:
     * - Something hasn't been initialized: `undefined`
     * - Something is current unavailable: `null`
     * Most other languages only have one (commonly called `null`). Since by default
     * JavaScript will evaluate an uninitialized variable/parameter/property to
     * `undefined` (you don't get a choice) we recommend you just use that for your
     * own unavailable status and don't bother with `null`.
     *
     * #### Example
     * ```typescript
     * isNull(null); // Returns `true`
     * isNull(undefined); // Returns `true`
     * isNull({}); // Returns `false`
     * ```
     *
     * @param pVariable Variable to validate.
     * @return Returns `true` if the variable is `null`.
     */
    export function isNull(pVariable: TAny): pVariable is null;
    /**
     * Validates if the supplied variable is an object and is available (not `null`
     * or `undefined`).
     *
     * #### Example
     * ```typescript
     * isObject({}); // Returns `true`
     * isObject([]); // Returns `false`
     * isObject(null); // Returns `false`
     * isObject(undefined); // Returns `false`
     * ```
     *
     * @param pObject Variable to validate.
     * @return Returns `true` if the variable is an object.
     */
    export function isObject(
        pObject: TAny
    ): pObject is {
        [property: string]: TAny;
    };
    /**
     * Validates if the supplied variable is a function.
     *
     * #### Example
     * ```typescript
     * isFunction(() => {}); // Returns `true`
     * ```
     *
     * @param pFunction Variable to validate.
     * @return Returns `true` if the variable is a function.
     */
    export function isFunction(
        pFunction: TAny
    ): pFunction is (...pArguments: TAny[]) => void | TAny;
    /**
     * Validates if the supplied variable is a string.
     *
     * #### Example
     * ```typescript
     * isString("1"); // Returns `true`
     * isString(1); // Returns `false`
     * ```
     *
     * @param pString Variable to validate.
     * @return Returns `true` if the variable is a string.
     */
    export function isString(pString: TAny): pString is string;
    /**
     * Returns the length of the supplied string.
     *
     * #### Example
     * ```typescript
     * stringLength("ABC"); // Returns `3`
     * stringLength(1); // Returns `0`
     * ```
     *
     * @param pString Specifies the string.
     * @return Returns the length of the string or `0` if the supplied variable is not a valid string or an empty string.
     */
    export function stringLength(pString: TAny): number;
    /**
     * Validates if the supplied variable is a string which is not empty.
     *
     * #### Example
     * ```typescript
     * isFilledString("ABC"); // Returns `true`
     * isFilledString(""); // Returns `false`
     * isFilledString(1); // Returns `false`
     * ```
     *
     * @param pString Variable to validate.
     * @return Returns `true` if the variable is a string with content.
     */
    export function isFilledString(pString: TAny): pString is string;
    /**
     * Validates if the supplied variable is a number (finite or infinite). To make sure the supplied number is a valid finite number, use
     * `isNumberFinite`.
     *
     * #### Example
     * ```typescript
     * isNumber(1); // Returns `true`
     * isNumber(Infinity); // Returns `true`
     * isNumber(NaN); // Returns `false`
     * isNumber("1"); // Returns `false`
     * ```
     *
     * @param pNumber Variable to validate.
     * @return Returns `true` if the variable is a number.
     */
    export function isNumber(pNumber: TAny): pNumber is number;
    /**
     * Validates if the supplied variable is a finite number and optionally checks if the number is within the specified range.
     *
     * #### Example
     * ```typescript
     * isNumberFinite(1); // Returns `true`
     * isNumberFinite(Infinity); // Returns `false`
     * isNumberFinite(NaN); // Returns `false`
     * isNumberFinite("1"); // Returns `false`
     * ```
     *
     * @param pNumber Variable to validate.
     * @param nRangeLower Optional parameter which specifies the lower range.
     * @param nRangeUpper Optional parameter which specifies the upper range.
     * @return Returns `true` if the variable is a number.
     */
    export function isNumberFinite(
        pNumber: TAny,
        nRangeLower?: number,
        nRangeUpper?: number
    ): pNumber is number;
    /**
     * Validates if the supplied variable is a floating point number and optionally checks if the number is within the specified range.
     *
     * #### Example
     * ```typescript
     * isFloat(1.01); // Returns `true`
     * isFloat(1); // Returns `false`
     * isFloat(Infinity); // Returns `false`
     * isFloat(NaN); // Returns `false`
     * isFloat("1"); // Returns `false`
     * ```
     *
     * @param pFloat Variable to validate.
     * @param fRangeLower Optional parameter which specifies the lower range.
     * @param fRangeUpper Optional parameter which specifies the upper range.
     * @return Returns `true` if the variable is a floating point number.
     */
    export function isFloat(
        pFloat: TAny,
        fRangeLower?: number,
        fRangeUpper?: number
    ): pFloat is number;
    /**
     * Validates if the supplied variable is a boolean.
     *
     * #### Example
     * ```typescript
     * isBoolean(false); // Returns `true`
     * isBoolean(1); // Returns `false`
     * ```
     *
     * @param pBool Variable to validate.
     * @return Returns `true` if the variable is a boolean.
     */
    export function isBoolean(pBool: TAny): pBool is boolean;
    /**
     * Validates if the supplied variable is an array.
     *
     * #### Example
     * ```typescript
     * isArray([]); // Returns `true`
     * isArray([1]); // Returns `true`
     * isArray(new Array()); // Returns `true`
     * isArray({}); // Returns `false`
     * ```
     *
     * @param pArray Variable to validate.
     * @return Returns `true` if the variable is an array.
     */
    export function isArray<T>(pArray: TAny): pArray is T[];
    /**
     * Validates if the supplied variable is a collection. A collection is a list
     * which is enumerable using a `length` property and an `item` function.
     * @param pCollection Variable to validate.
     * @return Returns `true` if the variable is a collection.
     */
    export function isCollection(pCollection: TAny): boolean;
    /**
     * Validates if the supplied variable is an array or a collection.
     * @param pArray Variable to validate.
     * @return Returns `true` if the variable is an array or collection.
     */
    export function isArrayOrCollection(pArray: TAny): boolean;
    /**
     * Returns the array size for the supplied array or the number of items in the object.
     *
     * #### Example
     * ```typescript
     * arraySize([]); // Returns `0`
     * arraySize([1, 2, 3]); // Returns `3`
     * ```
     *
     * @param pArray Specifies the array.
     * @return Returns the size of the array/object as a number.
     */
    export function arraySize(pArray: TAny): number;
    /**
     * Returns the item at the specified index in the array, collection or
     * enumerable object.
     *
     * #### Example
     * ```typescript
     * arrayItem<number>([1, 2, 3], 2); // Returns `3`
     * arrayItem<number>([1, 2, 3], 3); // Returns `undefined`
     * arrayItem<number>([1, 2, 3], 3, 0); // Returns `0`
     * ```
     *
     * @param T Specifies the item type.
     * @param pArray Specifies the array, collection or enumerable object.
     * @param nIndex Specifies the zero based item index.
     * @param pDefault Optional parameter which specifies the default value if the
     * supplied index is invalid.
     * @return Returns the item at the specified index.
     */
    export function arrayItem<T>(
        pArray: TList<T> | undefined,
        nIndex: number,
        pDefault?: T
    ): T | undefined;
    /**
     * Returns the first item in the array, collection or enumerable object.
     *
     * #### Example
     * ```typescript
     * firstArrayItem<number>([1, 2, 3]); // Returns `1`
     * ```
     *
     * @param T Specifies the item type.
     * @param pArray Specifies the array, collection or enumerable object.
     * @param pDefault Optional parameter which specifies the default value if there
     * is no first item.
     * @return Returns the last item.
     */
    export function firstArrayItem<T>(
        pArray: TList<T> | undefined,
        pDefault?: T
    ): T | undefined;
    /**
     * Returns the last item in the array, collection or enumerable object.
     *
     * #### Example
     * ```typescript
     * lastArrayItem<number>([1, 2, 3]); // Returns `3`
     * ```
     *
     * @param T Specifies the item type.
     * @param pArray Specifies the array, collection or enumerable object.
     * @param pDefault Optional parameter which specifies the default value if there
     * is no last item.
     * @return Returns the last item.
     */
    export function lastArrayItem<T>(
        pArray: TList<T> | undefined,
        pDefault?: T
    ): T | undefined;
    /**
     * Validates if the supplied variable is a date.
     *
     * #### Example
     * ```typescript
     * isDate(new Date()); // Returns `true`
     * ```
     *
     * @param pDate Variable to validate.
     * @return Returns `true` if the variable is a date.
     */
    export function isDate(pDate: TAny): pDate is Date;
    /**
     * Validates if the supplied variable is a regular expression.
     *
     * #### Example
     * ```typescript
     * isRegEx(/[a-z]+/); // Returns `true`
     * ```
     *
     * @param pRegEx Variable to validate.
     * @return Returns `true` if the variable is a regular expression.
     */
    export function isRegEx(pRegEx: TAny): boolean;
    /**
     * Validates if the supplied variable is an error object.
     *
     * #### Example
     * ```typescript
     * isError(new Error("Test message")); // Returns `true`
     * ```
     *
     * @param pError Variable to validate.
     * @return Returns `true` if the variable is an error object.
     */
    export function isError(pError: TAny): boolean;
    /**
     * Verifies if the supplied value is defined. If not, an error is thrown.
     * @param pAssert Specifies the value to assert.
     * @param sError Specifies the optional error message that is thrown.
     * @return Returns the value.
     */
    export function assert<T>(pAssert: T | undefined, sError?: string): T;
    /**
     * Cast variable to the desired type without type checking.
     * @param pSource Specifies the source.
     * @return Returns the desired type.
     */
    export function cast<To>(pSource: TAny): To;
    /**
     * Cast a variable to a string.
     *
     * #### Example
     * ```typescript
     * castToString(1); // Returns `1`
     * castToString(1.2); // Returns `1.2`
     * castToString(undefined); // Returns ``
     * ```
     *
     * @param pValue Source variable.
     * @param sDefault Optional parameter which specifies the default string if the supplied source variable cannot be casted.
     * @return Returns the string value.
     */
    export function castToString(pValue: TAny, sDefault?: string): string;
    /**
     * Cast a variable to a rounded number.
     *
     * #### Example
     * ```typescript
     * castToNumber("1"); // Returns `1`
     * castToNumber("1.5"); // Returns `2`
     * castToNumber(undefined); // Returns `0`
     * ```
     *
     * @param pValue Source variable.
     * @param nDefault Optional parameter which specifies the default number if the supplied source variable cannot be casted.
     * @return Returns the number value.
     */
    export function castToNumber(pValue: TAny, nDefault?: number): number;
    /**
     * Cast a variable to a floating point number.
     *
     * #### Example
     * ```typescript
     * castToFloat("1"); // Returns `1`
     * castToFloat("1.5"); // Returns `1.5`
     * castToFloat(undefined); // Returns `0`
     * ```
     *
     * @param pValue Source variable.
     * @param fDefault Optional parameter which specifies the default floating point number if the supplied source variable cannot be casted.
     * @return Returns the floating point number value.
     */
    export function castToFloat(pValue: TAny, fDefault?: number): number;
    /**
     * Cast a variable to a boolean value.
     *
     * #### Example
     * ```typescript
     * castToBoolean(1); // Returns `true`
     * castToBoolean("true"); // Returns `true`
     * castToBoolean("false"); // Returns `false`
     * ```
     *
     * @param pValue Source variable.
     * @param bDefault Optional parameter which specifies the default boolean value if the supplied source variable cannot be casted.
     * @return Returns the boolean value.
     */
    export function castToBoolean(pValue: TAny, bDefault?: boolean): boolean;
    /**
     * Clones an object.
     *
     * #### Example
     * ```typescript
     * clone({ a: 1, b: 2 }); // Returns `{ a: 1, b: 2 }`
     * ```
     *
     * @param pObject Specifies the object to clone.
     * @return Returns a clone of the object which is no reference to the original object.
     * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/create
     */
    export function clone<T extends IObject>(pObject: T): T;
    /**
     * Extend an object by appending all of the properties from each supplied
     * object. When there are identical properties, the right-most property takes
     * precedence. This function returns a mutable object reference which is the
     * supplied `pObject` (the first function argument).
     *
     * #### Example
     * ```typescript
     * extend({ a: 1, b: 2 }, { b: 3, d: 4 }, { d: 5 }); // Returns `{ a: 1, b: 3, d: 5 }`
     * ```
     *
     * @param pObject Specifies the object to extend.
     * @param pObjects Specifies the objects to extend with.
     * @return Returns the extended mutable object which is a reference to `pObject`.
     */
    export function extend<T extends IObject>(
        pObject: Partial<T>,
        ...pObjects: (Partial<T> | undefined)[]
    ): T;
    /**
     * Extend an object by appending all of the properties from each supplied
     * object. When there are identical properties, the right-most property takes
     * precedence. This function returns a immutable object reference which is
     * basically a new combined copy of the supplied objects.
     *
     * #### Example
     * ```typescript
     * extendImmutable({ a: 1, b: 2, c }, { b: 3, d: 4 }, { d: 5 }); // Returns `{ a: 1, b: 3, d: 5 }`
     * ```
     *
     * @param pObjects Specifies the objects to extend the immutable object with.
     * @return Returns the extended immutable object.
     */
    export function extendImmutable<T extends IObject>(
        ...pObjects: (Partial<T> | undefined)[]
    ): T;
    /**
     * Does a type or value comparison of two variables or a structure.
     *
     * #### Example
     * ```typescript
     * compare({ a: 1, b: 2 }, { a: 1, b: 3 }, false); // Returns `true`
     * compare({ a: 1, b: 2 }, { a: 1, c: 3 }, false); // Returns `false`
     * compare({ a: 1, b: 2 }, { a: 1, b: 3 }, true); // Returns `false`
     * compare({ a: 1, b: 2 }, { a: 1, b: 2 }, true); // Returns `true`
     * ```
     *
     * @param pA Specifies the first variable.
     * @param pB Specifies the second variable.
     * @param bCompareValues Specifies if the values or just the structure needs to be compared.
     * @return Returns `true` if the objects are identical.
     */
    export function compare(
        pA: TAny,
        pB: TAny,
        bCompareValues: boolean
    ): boolean;
    /**
     * Iterates through an array, collection or enumerable object and invokes the
     * supplied function for each element.
     *
     * #### Example
     * ```typescript
     * let nResult = 0;
     * each([1, 2, 3], (nItem: number) => nResult += nItem);
     * // The value of `nResult` is `6`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnCallee Specifies the function to be invoked for each element. The
     * element value will be exposed to the function as the first argument of the
     * argument list. Additional arguments can be specified and will be pushed to
     * the function.
     * @param fnCallee.pItem Contains the item.
     * @param fnCallee.pArguments Optional additional arguments. These are the
     * additional arguments which are passed to the main function.
     * @param pOptions Specifies the options.
     * @return Returns a reference to the list.
     */
    export function each<T>(
        pList: TList<T> | undefined,
        fnCallee: (pItem: T, ...pArguments: TArguments) => T | void,
        pOptions?: {
            /**
             * Specifies if each value in the array should be overwritten
             * with the return value of its executed function.
             */
            replace?: boolean;
            /**
             * Specifies if the item key should be passed as first argument in the
             * argument list of the function.
             */
            keys?: boolean;
            /**
             * Optional additional arguments which will be passed to the callee.
             */
            arguments?: TArguments;
        }
    ): TList<T> | undefined;
    /**
     * Iterates through an array, collection or enumerable object in reverse and
     * invokes the supplied function for each element.
     *
     * #### Example
     * ```typescript
     * let sResult = "";
     * eachReverse([1, 2, 3], (nItem: number) => nResult += CastToString(nItem));
     * // The value of `sResult` is `321`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnCallee Specifies the function to be invoked for each element. The
     * element value will be exposed to the function as the first argument of the
     * argument list. Additional arguments can be specified and will be pushed to
     * the function.
     * @param fnCallee.pItem Contains the item.
     * @param fnCallee.pArguments Optional additional arguments. These are the
     * additional arguments which are passed to the main function.
     * @param pOptions Specifies the options.
     * @return Returns a reference to the list.
     */
    export function eachReverse<T>(
        pList: TList<T> | undefined,
        fnCallee: (pItem: T, ...pArguments: TArguments) => T | void,
        pOptions?: {
            /**
             * Specifies if each value in the array should be overwritten
             * with the return value of its executed function.
             */
            replace?: boolean;
            /**
             * Specifies if the item key should be passed as first argument in the
             * argument list of the function.
             */
            keys?: boolean;
            /**
             * Optional additional arguments which will be passed to the callee.
             */
            arguments?: TArguments;
        }
    ): TList<T> | undefined;
    /**
     * Produces a new array of values by mapping each value in the supplied array,
     * collection or enumerable object through a transformation function.
     *
     * #### Example
     * ```typescript
     * map<number, number>([1, 2, 3], (nItem: number) => nItem * nItem); // Returns `[1, 4, 9]`
     * map<number, number>([1, 2, 3], (nItem: number, nMultiplier: number) => nItem * nMultiplier, 2); // Returns `[2, 4, 6]`
     * ```
     *
     * @param T Specifies the item type.
     * @param R Specifies the return type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTransformation Specifies the function to be invoked for each
     * element. The element value will be exposed to the function as the first
     * argument of the argument list. Additional arguments can be specified and will
     * be pushed to the function.
     * @param fnTransformation.pItem Contains the item.
     * @param fnTransformation.pArguments Optional additional arguments. These are
     * the additional arguments which are passed to the main function.
     * @param pArguments Optional additional arguments which will be passed to the
     * callee.
     * @return Returns an array with mapped values.
     */
    export function map<T, R>(
        pList: TList<T> | undefined,
        fnTransformation: (pItem: T, ...pArguments: TArguments) => R,
        ...pArguments: TArguments
    ): R[];
    /**
     * Reduce boils down an array, collection or enumerable object into a
     * single value of type `R`.
     *
     * #### Example
     * ```typescript
     * reduce<number, number>([1, 2, 3], (nValue: number, nItem: number) => nValue + nItem, 2); // Returns `8`
     * ```
     *
     * @param T Specifies the item type.
     * @param R Specifies the reduce type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnReduce Specifies the reduce function to be invoked for each
     * element. The return value of each reduce function will be the current
     * reduced value.
     * @param fnReduce.pValue Contains the current reduce value. If `pInitial` is
     * specified, this value is `pInitial` upon the first call otherwise this value
     * is `undefined`.
     * @param fnReduce.pItem Contains the item.
     * @param fnReduce.pArguments Optional additional arguments. These are the
     * additional arguments which are passed to the main function.
     * @param pInitial Optional initial value for the reduce value.
     * @param pArguments Optional additional arguments which will be passed to the
     * callee.
     * @return Returns the reduced value.
     */
    export function reduce<T, R>(
        pList: TList<T> | undefined,
        fnReduce: (pValue: R, pItem: T, ...pArguments: TArguments) => R,
        pInitial?: R,
        ...pArguments: TArguments
    ): R | undefined;
    /**
     * Verifies an array, collection or enumerable object by iterating through the
     * items and verify them against the supplied truth function.
     *
     * #### Example
     * ```typescript
     * has<number>([1, 2, 3], (nItem: number) => nItem > 0); // Returns `true`
     * has<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `false`
     * has<number>([1, 2, 3], (nItem: number) => nItem > 1, true); // Returns `true`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to
     * iterate through.
     * @param fnTruth Specifies the truth function. Each item is supplied to the
     * function, passing the item as argument. The function should return a boolean
     * value with the individual verification result.
     * @param fnTruth.pItem Contains the item.
     * @param bOneOrMore Optional parameter which specifies if one or more items
     * should pass the verification. By default (`false`) all items should
     * pass (equals `HasOnly`).
     * @return Returns `true` in case of a verification pass.
     */
    export function has<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean,
        bOneOrMore?: boolean
    ): boolean;
    /**
     * Verifies all items of an array, collection or enumerable object against the
     * truth function.
     *
     * #### Example
     * ```typescript
     * hasOnly<number>([1, 2, 3], (nItem: number) => nItem > 0); // Returns `true`
     * hasOnly<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `false`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the truth function. Each item is supplied to the
     * function, passing the item as argument. The function should return a boolean
     * value with the individual verification result.
     * @param fnTruth.pItem Contains the item.
     * @return Returns `true` when all items pass the verification test.
     */
    export function hasOnly<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean
    ): boolean;
    /**
     * Verifies some items of an array, collection or enumerable object against the
     * truth function.
     *
     * #### Example
     * ```typescript
     * hasOneOrMore<number>([1, 2, 3], (nItem: number) => nItem > 2); // Returns `true`
     * hasOneOrMore<number>([1, 2, 3], (nItem: number) => nItem > 3); // Returns `false`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the truth function. Each item is supplied to the
     * function, passing the item as argument. The function should return a boolean
     * value with the individual verification result.
     * @param fnTruth.pItem Contains the item.
     * @return Returns `true` when one or more items pass the verification test.
     */
    export function hasOneOrMore<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean
    ): boolean;
    /**
     * Counts the number of items in the supplied array, collection or
     * enumerable object.
     *
     * #### Example
     * ```typescript
     * count<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `2`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the optional truth function. Only items that pass
     * the truth test are included in the count. The function should return a
     * boolean value whether the item should be counted. If omitted all items are
     * counted.
     * @param fnTruth.pItem Contains the item.
     * @return Returns the number of items that pass.
     */
    export function count<T>(
        pList: TList<T> | undefined,
        fnTruth?: (pItem: T) => boolean
    ): number;
    /**
     * Selects items from an array, collection or enumerable object using a truth
     * test and optionally invokes a function for each item.
     *
     * #### Example
     * ```typescript
     * forEach<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `[2, 3]`
     * forEach<number>([1, 2, 3], (nItem: number) => nItem > 1, {
     *   do: (nItem: number) => nItem + 1,
     *   replace: true
     * }); // Returns `[3, 4]`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the optional selector function. Each item is
     * supplied to the function, passing the item as argument. The function should
     * return a boolean value whether to include the item in the selection. If
     * omitted all items are selected.
     * @param fnTruth.pItem Contains the item.
     * @param pOptions Contains the options.
     * @return Returns an array with the selected items or just the selected item if
     * the single mode is enabled.
     */
    export function forEach<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean,
        pOptions?: {
            /**
             * Specifies an optional function to be invoked for each element that
             * passed the truth test. The element value will be exposed to the function
             * as the first argument of the argument list. Additional arguments can be
             * specified and will be pushed to the function.
             * @param pItem Contains the item.
             * @param pArguments Optional additional arguments. These are the
             * additional arguments which are passed to the main function.
             */
            do?: (pItem: T, ...pArguments: TArguments) => T | void;
            /**
             * Specifies if each value in the array should be overwritten  with the
             * return value of its executed function.
             */
            replace?: boolean;
            /**
             * Specifies if the function should return items. The default setting
             * is `matches` which returns all items which passed the truth test.
             */
            return?: "none" | "matches" | "first" | "last";
            /** Optional arguments which will be passed to the optional call function. */
            arguments?: TArguments;
        }
    ): T[] | T | undefined;
    /**
     * Selects all items from an array, collection or enumerable object that match
     * the truth function.
     *
     * #### Example
     * ```typescript
     * filter<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `[2, 3]`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the optional truth function. Each item is supplied
     * to the function, passing the item as argument. The function should return a
     * boolean value whether to include the item in the selection. If omitted all
     * items are selected.
     * @param fnTruth.pItem Contains the item.
     * @return Returns an array with the selected items.
     */
    export function filter<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean
    ): T[];
    /**
     * Selects the first item from an array, collection or enumerable object that
     * matches the truth function.
     *
     * #### Example
     * ```typescript
     * findFirst<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `2`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the optional truth function. Each item is supplied
     * to the function, passing the item as argument. The function should return a
     * boolean value whether to include the item in the selection. If omitted all
     * items match.
     * @param fnTruth.pItem Contains the item.
     * @return Returns the item or `undefined` if no item is found.
     */
    export function findFirst<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean
    ): T | undefined;
    /**
     * Selects the last item from an array, collection or enumerable object that
     * matches the truth function.
     *
     * #### Example
     * ```typescript
     * findLast<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `3`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the optional truth function. Each item is supplied
     * to the function, passing the item as argument. The function should return a
     * boolean value whether to include the item in the selection. If omitted all
     * items match.
     * @param fnTruth.pItem Contains the item.
     * @return Returns the item or `undefined` if no item is found.
     */
    export function findLast<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean
    ): T | undefined;
    /**
     * Finds the first item from an array, collection or enumerable object that
     * matches the truth function and returns its key.
     *
     * #### Example
     * ```typescript
     * find<number>([1, 2, 3], (nItem: number) => nItem > 1); // Returns `1`
     * find<Object>({a: 1, b: 2, c: 3}, (nItem: number) => nItem > 1); // Returns `b`
     * ```
     *
     * @param T Specifies the item type.
     * @param pList Specifies the array, collection or enumerable object to iterate
     * through.
     * @param fnTruth Specifies the optional truth function. Each item is supplied
     * to the function, passing the item as argument. The function should return a
     * boolean value whether to include the item in the selection. If omitted all
     * items match.
     * @param fnTruth.pItem Contains the item.
     * @return Returns the item key or `undefined` if no item is found.
     */
    export function find<T>(
        pList: TList<T> | undefined,
        fnTruth: (pItem: T) => boolean
    ): string | number | undefined;
    /**
     * Calls the supplied function.
     * @param T Specifies the return type of the call.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns `undefined` if the call fails or the return value of the function which is executed if the call succeeded.
     */
    export function call<T>(
        fnCallee: ((...pArguments: TArguments) => T | void) | undefined,
        ...pArguments: TArguments
    ): T | undefined;
    /**
     * Calls the supplied function asynchronous. Tries the following schedulers in
     * this order: `FUNCTION_TICK`, `FUNCTION_TIMEOUT`. The call cannot be canceled.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function callAsync(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): void;
    /**
     * Calls the supplied function using promise. Tries the following schedulers in
     * this order: `FUNCTION_PROMISE`, `FUNCTION_TICK`, `FUNCTION_TIMEOUT`. The call cannot be canceled.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function callPromise(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): void;
    /**
     * Calls the supplied function synchronous or asynchronous.
     * @param fnCallee Specifies the function to execute.
     * @param bSynchronous Specifies if the functio should be executed synchronously.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle for asynchronous calls or `0` for synchronous calls.
     */
    export function callEx(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        bSynchronous: boolean,
        ...pArguments: TArguments
    ): void;
    /**
     * Calls the supplied function and binds the specified context.
     * @param T Specifies the return type of the call.
     * @param fnCallee Specifies the function to execute.
     * @param pContext Context to bind to.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns `undefined` if the call fails or the return value of the function which is executed if the call succeeded.
     */
    export function callBind<T>(
        fnCallee: ((...pArguments: TArguments) => T | void) | undefined,
        pContext: {},
        ...pArguments: TArguments
    ): T | undefined;
    /**
     * Calls the supplied function.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns `undefined` if the call fails or the return value of the function which is executed if the call succeeded.
     */
    export function callFunction(
        fnCallee: TAny,
        ...pArguments: TArguments
    ): TAny;
    /**
     * Calls the supplied function asynchronous on the next tick.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleTick(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): number;
    /**
     * Calls the supplied function asynchronous on the next tick and cancels a pending call.
     * @param fnCallee Specifies the function to execute.
     * @param nHandle Specifies the handle of the call to cancel.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleAndCancelTick(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        nHandle: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels a pending asynchronous tick call.
     * @param nHandle Contains the handle of the call.
     * @return Always returns 0.
     */
    export function cancelTick(nHandle: number): number;
    /**
     * Calls the supplied function as a promise.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function schedulePromise(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): number;
    /**
     * Calls the supplied function as a promise and cancels a pending promise.
     * @param fnCallee Specifies the function to execute.
     * @param nHandle Specifies the handle of the call to cancel.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleAndCancelPromise(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        nHandle: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels a pending promise.
     * @param nHandle Contains the handle of the call.
     * @return Always returns 0.
     */
    export function cancelPromise(nHandle: number): number;
    /**
     * Calls the supplied function asynchronous on the next frame.
     * @param fnCallee Specifies the function to execute.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleFrame(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): number;
    /**
     * Calls the supplied function asynchronous on the next frame.
     * @param fnCallee Specifies the function to execute.
     * @param nHandle Specifies the handle of the call to cancel.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleAndCancelFrame(
        fnCallee: ((...pArguments: TArguments) => void) | undefined,
        nHandle: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels a pending asynchronous frame call.
     * @param nHandle Contains the handle of the call.
     * @return Always returns 0.
     */
    export function cancelFrame(nHandle: number): number;
    /**
     * Schedules the supplied function and invokes it on the specified timeout (using `setTimeout`).
     * @param fnCallee Specifies the function to execute.
     * @param nTimeout Specifies the timeout in milliseconds.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleTimeout(
        fnCallee: (...pArguments: TArguments) => void,
        nTimeout: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels a pending asynchronous call.
     * @param nHandle Contains the handle of the asynchronous timeout call.
     * @return Always returns 0.
     */
    export function cancelTimeout(nHandle: number): number;
    /**
     * Schedules the supplied function and invokes it on the specified timeout
     * (using `requestAnimationFrame`) which is optimal for UI related timers.
     * @param fnCallee Specifies the function to execute.
     * @param nTimeout Specifies the timeout in milliseconds.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     * @see https://bugs.chromium.org/p/chromium/issues/detail?id=574343
     */
    export function scheduleUITimeout(
        fnCallee: (...pArguments: TArguments) => void,
        nTimeout: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels a pending UI timeout call.
     * @param nHandle Contains the handle of the asynchronous timeout call.
     * @return Always returns 0.
     */
    export function cancelUITimeout(nHandle: number): number;
    /**
     * Schedules the supplied function and invokes it on the specified interval (using `setInterval`).
     * @param fnCallee Specifies the function to execute.
     * @param nInterval Specifies the interval in milliseconds.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     */
    export function scheduleInterval(
        fnCallee: (...pArguments: TArguments) => void,
        nInterval: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels an interval call.
     * @param nHandle Contains the handle of the asynchronous interval call.
     * @return Always returns 0.
     */
    export function cancelInterval(nHandle: number): number;
    /**
     * Schedules the supplied function and invokes it on the specified interval
     * (using `requestAnimationFrame`) which is optimal for UI related timers.
     * @param fnCallee Specifies the function to execute.
     * @param nInterval Specifies the interval in milliseconds.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     * @return Returns the call handle.
     * @see https://bugs.chromium.org/p/chromium/issues/detail?id=574343
     */
    export function scheduleUIInterval(
        fnCallee: (...pArguments: TArguments) => void,
        nInterval: number,
        ...pArguments: TArguments
    ): number;
    /**
     * Cancels a pending UI interval call.
     * @param nHandle Contains the handle of the asynchronous interval call.
     * @return Always returns 0.
     */
    export function cancelUIInterval(nHandle: number): number;
    /**
     * Calls the supplied function at a specific number of frames.
     * @param fnAnimation Specifies the function to execute.
     * @param fnAnimation.pArguments Optional additional arguments. These are the additional arguments which are passed to the main function.
     * @param nFrames Specifies the number of frames.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     */
    export function scheduleNumberOfFrames(
        fnAnimation: ((...pArguments: TArguments) => void) | undefined,
        nFrames: number,
        ...pArguments: TArguments
    ): void;
    /**
     * Calls the supplied animation function using a double animation frame request. Use this function if you need to be sure all pending
     * DOM reads and writes are performed.
     * @param fnAnimation Specifies the function to execute.
     * @param fnAnimation.pArguments Optional additional arguments. These are the additional arguments which are passed to the main function.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     */
    export function scheduleAnimation(
        fnAnimation: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): void;
    /**
     * Calls the supplied event function (after 2 animation frames).
     * @param fnEvent Specifies the function to execute.
     * @param fnEvent.pArguments Optional additional arguments. These are the additional arguments which are passed to the main function.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     */
    export function scheduleEvent(
        fnEvent: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): void;
    /**
     * Calls the supplied function as part of an action in a waiting loop (after 5 animation frames).
     * @param fnAction Specifies the function to execute.
     * @param fnAction.pArguments Optional additional arguments. These are the additional arguments which are passed to the main function.
     * @param pArguments Optional additional arguments which will be passed to the callee.
     */
    export function scheduleAction(
        fnAction: ((...pArguments: TArguments) => void) | undefined,
        ...pArguments: TArguments
    ): void;
    /**
     * Sets the value of the specified variable in the object using its index type.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @param pValue Value for the variable.
     * @return Returns a reference to the value on the object.
     */
    export function set<T, K extends keyof T>(
        pObject: T,
        sName: K,
        pValue: T[K]
    ): T[K];
    /**
     * Sets the value of the specified variable in the object array.
     * @param T Specifies the type of the variable which is set.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @param pValue Value for the variable.
     * @return Returns a reference to the value on the object.
     */
    export function setAny<T>(pObject: TAny, sName: string, pValue: T): T;
    /**
     * Retrieves the value of the specified variable in the object using its index type.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @return Returns the value of the variable.
     */
    export function get<T, K extends keyof T>(pObject: T, sName: K): T[K];
    /**
     * Retrieves the value of the specified variable in the object array.
     * @param T Specifies the type of the returned variable.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @return Returns the value of the variable.
     */
    export function getAny<T>(
        pObject: IObject | undefined,
        sName: string
    ): T | undefined;
    /**
     * Retrieves a boolean value of the specified variable in the object array.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @param bDefault Specifies the default value.
     * @return Returns the value of the variable.
     */
    export function getBoolean(
        pObject: IObject | undefined,
        sName: string,
        bDefault?: boolean
    ): boolean;
    /**
     * Retrieves a string value of the specified variable in the object array.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @param sDefault Specifies the default value.
     * @return Returns the value of the variable.
     */
    export function getString(
        pObject: IObject | undefined,
        sName: string,
        sDefault?: string
    ): string;
    /**
     * Retrieves a numeric value of the specified variable in the object array.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @param nDefault Specifies the default value.
     * @return Returns the value of the variable.
     */
    export function getNumber(
        pObject: IObject | undefined,
        sName: string,
        nDefault?: number
    ): number;
    /**
     * Retrieves a floating value of the specified variable in the object array.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @param fDefault Specifies the default value.
     * @return Returns the value of the variable.
     */
    export function getFloat(
        pObject: IObject | undefined,
        sName: string,
        fDefault?: number
    ): number;
    /**
     * Destroys the specified variable in the object array.
     * @param pObject Reference to the object.
     * @param sName Name of the variable.
     * @return Returns `true` if the variable is deleted.
     */
    export function destroy(
        pObject: IObject | undefined,
        sName: string
    ): boolean;
    /**
     * Creates a new class instance with the supplied constructor and arguments
     * array.
     *
     * #### Example
     * ```typescript
     * class Example {
     *     constructor(public a: string, public b: number) {}
     * }
     *
     * let pExample = createInstanceOf(Example, "Hello", 1);
     * ```
     *
     * @param T Specifies the instance type.
     * @param pConstructor Specifies the constructor.
     * @param pArguments Optional array with arguments which will be passed to the constructor.
     * @return Returns the instance.
     */
    export function createInstanceOf<T>(
        pConstructor: {
            new (...pArguments: TArguments): T;
        },
        ...pArguments: TArguments
    ): T;
}

declare module "@marblecore/functional/lib/console" {
    export namespace Console {
        /**
         * Returns a log string with a time stamp.
         * @return Returns the time stamp log string.
         */
        function time(): string;
        /**
         * Returns a log string for a module name.
         * @param sName Specifies the name of the module.
         * @return Returns the log string.
         */
        function module(sName: string): string;
        /**
         * Returns a complete log string.
         * @param sName Specifies the name of the module.
         * @param sMessage Specifies the message to log.
         * @return Returns the log string.
         */
        function log(sName: string, sMessage: string): string;
    }
}

declare module "@marblecore/functional/lib/environment" {
    export namespace Environment {
        /** Contains if the environment is a browser. */
        const isBrowser: boolean;
        /** Contains if the environment is Node. */
        const isNode: boolean;
        /** Contains if the environment is a web worker. */
        const isWebWorker: boolean;
    }
}

declare module "@marblecore/functional/lib/number" {
    export namespace Num {
        /**
         * Round a floating point number to the nearest integer.
         *
         * #### Example
         * ```typescript
         * round(1.49); // Returns `1`
         * round(1.5); // Returns `2`
         * ```
         *
         * @param fValue Input number.
         * @return Returns the rounded number.
         */
        function round(fValue: number): number;
        /**
         * Round a floating point number downward to its nearest integer.
         *
         * #### Example
         * ```typescript
         * floor(1.6); // Returns `1`
         * ```
         *
         * @param fValue Input number.
         * @return Returns the floored number.
         */
        function floor(fValue: number): number;
        /**
         * Round a floating point number upward to its nearest integer.
         *
         * #### Example
         * ```typescript
         * ceil(1.6); // Returns `2`
         * ```
         *
         * @param fValue Input number.
         * @return Returns the ceiled number.
         */
        function ceil(fValue: number): number;
        /**
         * Conforms the supplied number to the specified precision.
         *
         * #### Example
         * ```typescript
         * conform(1.1235, 3); // Returns `1.124`
         * conform(1.1235, 2); // Returns `1.12`
         * ```
         *
         * @param fValue Input value.
         * @param nPrecision Specifies the number of decimals.
         * @return Returns the conformed number.
         */
        function conform(fValue: number, nPrecision: number): number;
        /**
         * Checks if the supplied number is positive and returns this positive value or `0` if the value is negative.
         *
         * #### Example
         * ```typescript
         * positive(1); // Returns `1`
         * positive(-1); // Returns `0`
         * ```
         *
         * @param nValue Input number.
         * @return Returns nValue if the value is positive otherwise `0`.
         */
        function positive(nValue: number): number;
        /**
         * Checks if the supplied number is negative and returns this negative value or `0` if the value is positive.
         *
         * #### Example
         * ```typescript
         * negative(1); // Returns `0`
         * negative(-1); // Returns `-1`
         * ```
         *
         * @param nValue Input number.
         * @return Returns nValue if the value is negative otherwise `0`.
         */
        function negative(nValue: number): number;
        /**
         * Compares two numbers and returns the highest value.
         *
         * #### Example
         * ```typescript
         * max(1, 2); // Returns `2`
         * ```
         *
         * @param nA Input number A.
         * @param nB Input number B.
         * @return Returns the number with the highest value.
         */
        function max(nA: number, nB: number): number;
        /**
         * Compares the supplied arguments returns the highest value.
         *
         * #### Example
         * ```typescript
         * maxL(1, 2, 5, 3); // Returns `5`
         * ```
         *
         * @param pArguments Arguments to compare.
         * @return Returns the number with the highest value.
         */
        function maxL(...pArguments: number[]): number;
        /**
         * Compares two numbers and returns the lowest value.
         *
         * #### Example
         * ```typescript
         * min(1, 2); // Returns `1`
         * ```
         *
         * @param nA Input number A.
         * @param nB Input number B.
         * @return Returns the number with the lowest value.
         */
        function min(nA: number, nB: number): number;
        /**
         * Compares the supplied arguments returns the lowest value.
         *
         * #### Example
         * ```typescript
         * minL(1, 2, 5, 3); // Returns `1`
         * ```
         *
         * @param pArguments Arguments to compare.
         * @return Returns the number with the lowest value.
         */
        function minL(...pArguments: number[]): number;
        /**
         * Adjusts a number so it is in range between the specifies minimum and maximum.
         *
         * #### Example
         * ```typescript
         * range(20, 5, 10); // Returns `10`
         * range(0, 5, 10); // Returns `5`
         * ```
         *
         * @param nValue Input number.
         * @param nMin Specifies the minimum value.
         * @param nMax Specifies the maximum value.
         * @return Returns the ranged number.
         */
        function range(nValue: number, nMin: number, nMax?: number): number;
        /**
         * Checks if the given value is within the specified range.
         *
         * #### Example
         * ```typescript
         * inRange(5, 5, 10); // Returns `true`
         * inRange(0, 5, 10); // Returns `false`
         * ```
         *
         * @param nValue Input number.
         * @param nMin Specifies the minimum value.
         * @param nMax Specifies the maximum value.
         * @param bEdgeMin Specifies if the edge of the min value is allowed (enabled by default).
         * @param bEdgeMax Specifies if the edge of the max value is allowed (disabled by default).
         * @return Returns `true` if the value is in range.
         */
        function inRange(
            nValue: number,
            nMin: number,
            nMax: number,
            bEdgeMin?: boolean,
            bEdgeMax?: boolean
        ): boolean;
        /**
         * Formats a number by inserting thousands and decimal separators while taking rounding into account.
         *
         * #### Example
         * ```typescript
         * format("1000.235", 2); // Returns `1,000.24`
         * ```
         *
         * @param nValue Input value as a number or string.
         * @param nPrecision Specifies the precision.
         * @param sSeparator Separator sign.
         * @param sDecimal Decimal sign.
         * @param sMinus Minus sign.
         * @return Returns the formatted number as a string.
         */
        function format(
            nValue: number | string,
            nPrecision?: number,
            sSeparator?: string,
            sDecimal?: string,
            sMinus?: string
        ): string;
    }
}

declare module "@marblecore/functional/lib/string" {
    import { TArguments, TList } from "@marblecore/functional/lib/types";
    export namespace Str {
        /**
         * Converts a string to lowercase.
         *
         * #### Example
         * ```typescript
         * lowercase("HELLO"); // Returns `hello`
         * ```
         *
         * @param sValue Source string.
         * @return Returns the converted string.
         */
        function lowercase(sValue: string): string;
        /**
         * Converts a string to uppercase.
         *
         * #### Example
         * ```typescript
         * uppercase("Hello"); // Returns `HELLO`
         * ```
         *
         * @param sValue Source string.
         * @return Returns the converted string.
         */
        function uppercase(sValue: string): string;
        /**
         * Capitalizes the first character of a string or the first character of each word.
         *
         * #### Example
         * ```typescript
         * capitalize("lorem ipsum. dolor"); // Returns `Lorem ipsum. dolor`
         * capitalize("lorem ipsum. dolor", "each-word"); // Returns `Lorem Ipsum. Dolor`
         * capitalize("lorem ipsum. dolor", "each-sentence"); // Returns `Lorem ipsum. Dolor`
         * ```
         *
         * @param sValue Source string.
         * @param sMode Specifies the mode: `first-character` (default), `each-word` or `each-sentence`.
         * @param bLowercase Converts the string to lowercase before capitalizing it.
         * @return Returns the capitalized string.
         */
        function capitalize(
            sValue: string,
            sMode?: "first-character" | "each-word" | "each-sentence",
            bLowercase?: boolean
        ): string;
        /**
         * Trims a string by removing all leading and trailing whitespaces (`_ab__cd_` -> `ab__cd`).
         *
         * #### Example
         * ```typescript
         * trim(" ab  cd "); // Returns `ab  cd`
         * ```
         *
         * @param sValue Source string.
         * @return Returns the trimmed string.
         */
        function trim(sValue: string): string;
        /**
         * Trims a string at the left side by removing all leading whitespaces (`_ab__cd_` -> `ab__cd_`).
         *
         * #### Example
         * ```typescript
         * trimLeft(" ab  cd "); // Returns `ab  cd `
         * ```
         *
         * @param sValue Source string.
         * @return Returns the trimmed string.
         */
        function trimLeft(sValue: string): string;
        /**
         * Trims a string at the right by removing all trailing whitespaces (`_ab__cd_` -> `_ab__cd`).
         *
         * #### Example
         * ```typescript
         * trimRight(" ab  cd "); // Returns ` ab  cd`
         * ```
         *
         * @param sValue Source string.
         * @return Returns the trimmed string.
         */
        function trimRight(sValue: string): string;
        /**
         * Trims a string by removing all multiple whitespaces (`_ab__cd_` -> `_ab_cd_`).
         *
         * #### Example
         * ```typescript
         * trimMultiple(" ab  cd "); // Returns ` ab cd `
         * ```
         *
         * @param sValue Source string.
         * @return Returns the trimmed string.
         */
        function trimMultiple(sValue: string): string;
        /**
         * Sanitize a string by removing all leading, trailing and multiple whitespaces (`_ab__cd_` -> `ab_cd`).
         *
         * #### Example
         * ```typescript
         * sanitize(" ab  cd "); // Returns `ab cd`
         * ```
         *
         * @param sValue Source string.
         * @return Returns the sanitized string.
         */
        function sanitize(sValue: string): string;
        /**
         * Removes all white spaces from the specified string (`_ab__cd_` -> `abcd`).
         *
         * #### Example
         * ```typescript
         * removeWhitespaces(" ab  cd "); // Returns `abcd`
         * ```
         *
         * @param sValue Source string.
         * @return Returns the sanitized string.
         */
        function removeWhitespaces(sValue: string): string;
        /**
         * Limits a string which is greater than the specified number of characters and appends an optional string. If an append string is
         * specified any trailing spaces in the trimmed string are removed.
         *
         * #### Example
         * ```typescript
         * limit("Lorem ipsum", 5, "..."); // Returns `Lorem...`
         * limit("Lorem ipsum", 6, "..."); // Returns `Lorem...`
         * limit("Lorem ipsum", 6); // Returns `Lorem `
         * ```
         *
         * @param sValue Specifies the input string (variable will be casted to a string if necessary).
         * @param nMax Maximum string length.
         * @param sAppend Optional string which is appended to a limited string.
         * @return Returns the limited string.
         */
        function limit(sValue: string, nMax: number, sAppend?: string): string;
        /**
         * Replaces all occurrences of `sWhat` with `sWith` in the specified string.
         *
         * #### Example
         * ```typescript
         * replace("Hello", "l", "-"); // Returns `He--o`
         * ```
         *
         * @param sValue Specifies the input string (variable will be casted to a string if necessary).
         * @param sWhat Specifies the string to search for.
         * @param sWith Specifies the replace string. If omitted an empty string will be used.
         * @param bIgnoreCase Specifies if the string replace should be case insensitive.
         * @return Returns the replaced string.
         */
        function replace(
            sValue: string,
            sWhat: string,
            sWith?: string,
            bIgnoreCase?: boolean
        ): string;
        /**
         * Replaces all occurrences of strings in `pWhat` with `sWith` in the specified string.
         *
         * #### Example
         * ```typescript
         * replaceMultiple("Hello", ["ll", "o"], "-"); // Returns `He---`
         * ```
         *
         * @param sValue Specifies the input string (variable will be casted to a string if necessary).
         * @param pWhat Specifies the strings to search for.
         * @param sWith Specifies the replace string. If omitted an empty string will be used.
         * @param bIgnoreCase Specifies if the string replace should be case insensitive.
         * @return Returns the replaced string.
         */
        function replaceMultiple(
            sValue: string,
            pWhat: string[],
            sWith?: string,
            bIgnoreCase?: boolean
        ): string;
        /**
         * Converts HTML brackets to HTML entities so it is safe to display without HTML parsing. *Careful, only the brackets (`<` and `>`)
         * are converted to HTML entities.*
         *
         * #### Example
         * ```typescript
         * makeHTMLSafe("<b>Hello</b>"); // Returns `&lt;b&gt;Hello&lt;b&gt;`
         * ```
         *
         * @param sValue Specifies the string to be converted (variable will be casted to a string if necessary).
         * @return Returns the converted string.
         */
        function makeHTMLSafe(sValue: string): string;
        /**
         * Converts carriage returns and/or newlines to HTML breaks.
         *
         * #### Example
         * ```typescript
         * CRLFToHTML("Row 1\r\nRow 2"); // Returns `Row 1<br />Row 2`
         * ```
         *
         * @param sValue Specifies the string to be converted (variable will be casted to a string if necessary).
         * @return Returns the converted string.
         */
        function CRLFToHTML(sValue: string): string;
        /**
         * Converts an array or object list to a concatenated string.
         *
         * #### Example
         * ```typescript
         * iterateToString(["a", "b", "c"]); // Returns `abc`
         * iterateToString(["a", "b", "c"], "-"); // Returns `a-b-c`
         * iterateToString(["a", "b", "c"], "/", (sValue: string) => sValue + sValue); // Returns `aa/bb/cc`
         * ```
         *
         * @param T Specifies the item type.
         * @param pList Specifies the array, collection or enumerable object to
         * iterate through.
         * @param sSeparator Optional parameter which specifies the separator string or char.
         * @param fnCast Optional function to be invoked for each element instead of automatic casting to strings. The element value will be
         * exposed to the function as the first argument of the argument list. Additional arguments can be specified and will be pushed to
         * the function.
         * @param fnCast.pItem Contains the item which should be casted to a string.
         * @param fnCast.pArguments Optional additional arguments for the casting function. These are the additional arguments which are
         * passed to the main function.
         * @param pArguments Optional additional arguments which will be passed to the callee.
         * @return Returns the concatenated string.
         */
        function iterateToString<T>(
            pList: TList<T>,
            sSeparator?: string,
            fnCast?: (pItem: T, ...pArguments: TArguments) => string,
            ...pArguments: TArguments
        ): string;
        /**
         * Extracts the part of the string between the first occurrence of `left` and the optional occurrence of `right`.
         *
         * #### Example
         * ```typescript
         * extract("Lorem ipsum dolor", { left: "Lo", right: "m" }); // Returns `re`
         * extract("Lorem ipsum dolor", { left: "Lo", right: "m", fromEnd: true }); // Returns `rem ipsu`
         * ```
         *
         * @param sValue Specifies the input string (variable will be casted to a string if necessary).
         * @param pOptions Specifies the extract options.
         * @return Returns the string between sLeft and sRight or an empty string if an error occurs.
         */
        function extract(
            sValue: string,
            pOptions: {
                /** Left string selector. */
                left: string;
                /** Right string selector. */
                right?: string;
                /**
                 * Specifies the search direction of the right string selector. If
                 * `false` is specified, the function searches for the first
                 * occurrence of `Right` directly after the position of `Left`.
                 */
                fromEnd?: boolean;
                /** Specifies if a case insensitive check should be performed. */
                ignoreCase?: boolean;
            }
        ): string;
        /**
         * Fills a string for the specified times with the specified string or characters.
         *
         * #### Example
         * ```typescript
         * fill("A", 5); // Returns `AAAAA`
         * ```
         *
         * @param sFill Specifies the fill string.
         * @param nCount Specifies the number of copies to insert.
         * @return Returns the filled string.
         */
        function fill(sFill: string, nCount: number): string;
        /**
         * Converts source variable to a string and pads the string on the left with the supplied number of characters.
         *
         * #### Example
         * ```typescript
         * padLeft("ABC", "A", 5); // Returns `AAABC`
         * ```
         *
         * @param pValue Specifies the input string or number.
         * @param sFill Contains the string which will be used as fill string.
         * @param nLength Specifies the desired string length.
         * @param bCrop Optional boolean value which enables string cropping if the source string length is larger than the desired string
         * length.
         * @param bTreatAsNumber Optional boolean value which specifies the input string should be treated as a number.
         * @return Returns the padded string.
         */
        function padLeft(
            pValue: string | number,
            sFill: string,
            nLength: number,
            bCrop?: boolean,
            bTreatAsNumber?: boolean
        ): string;
        /**
         * Converts source variable to a string and pads the string on the right with the supplied number of characters.
         *
         * #### Example
         * ```typescript
         * padRight("ABC", "A", 5); // Returns `ABCAA`
         * ```
         *
         * @param pValue Specifies the input string or number.
         * @param sFill Contains the string which will be used as fill string.
         * @param nLength Specifies the desired string length.
         * @param bCrop Optional boolean value which enables string cropping if the source string length is larger than the desired string
         * length.
         * @return Returns the padded string.
         */
        function padRight(
            pValue: string | number,
            sFill: string,
            nLength: number,
            bCrop?: boolean
        ): string;
        /**
         * Creates a simple hash for the supplied string using the djb2-algorithm
         * written by Dan Bernstein. This hash function is similar to a linear
         * congruential generator and is absolutely not collision resistant.
         *
         * #### Example
         * ```typescript
         * djb2Hash("Abc"); // Returns `ABCAA`
         * djb2Hash("Abc", "prefix-"); // Returns `prefix-ABCAA`
         * ```
         *
         * @param sValue Specifies the input string.
         * @param sPrefix Specifies a prefix for the hash result.
         * @return Returns the hash string.
         * @see http://www.cse.yorku.ca/~oz/hash.html
         * @see https://en.wikipedia.org/wiki/Linear_congruential_generator
         */
        function djb2Hash(sValue: string, sPrefix?: string): string;
    }
}

declare module "@marblecore/functional/lib/types" {
    export type TAny = any;
    export type TArguments = TAny[];
    /** Describes an object. */
    export interface IObject {
        [name: string]: TAny;
    }
    /** Describes a collection. */
    export interface ICollection<T> {
        /** Contains the number of items in the collection. */
        length: number;
        /** Retrieves the item at the specified index. */
        item(nIndex: number): T;
    }
    /** Describes an enumerable object. */
    export interface IEnumerable<T> {
        [name: string]: T;
    }
    /** Type definition for lists (arrays, collections or enumerable objects). */
    export type TList<T> = T[] | ICollection<T> | IEnumerable<T>;
}

declare module "@marblecore/functional/lib/date" {
    export class DateTime {
        /** Returns the current time and date as a number. */
        static readonly now: number;
        /** Returns a high precision time stamp with up to microsecond precision. */
        static readonly precise: number;
        /**
         * Checks if the specified date/time is in the future.
         * @param nTime Specifies the date/time to check.
         * @return Returns `true` if the supplied date/time is in the future.
         */
        static isFuture(nTime: number): boolean;
        /**
         * Checks if the specified date/time is in the past.
         * @param nTime Specifies the date/time to check.
         * @return Returns `true` if the supplied date/time is in the past.
         */
        static isPast(nTime: number): boolean;
        /**
         * Returns the number of elapsed milliseconds between the current time and the supplied time.
         * @param nTime Specifies the date/time to subtract.
         * @param bPrecise Specifies if the precise or normal time should be used.
         * @return Returns the number of elapsed milliseconds between the current time and the supplied time.
         */
        static elapsed(nTime: number, bPrecise?: boolean): number;
        /**
         * Resets the high precision timer and returns the old value.
         * @return Returns the previous timer value.
         */
        static timerReset(): number;
        /** Returns the number of milliseconds elapsed since the last timer update. On each call the timer is restarted. */
        static readonly timerElapsed: number;
        /** Returns a timer stamp with the number of milliseconds elapsed since the last timer update. */
        static readonly timerStamp: string;
        /**
         * Formats a date/time string.
         * @param sFormat Specifies the format to use, using the CLDR date/time formatting style. The following variables are available:
         * - `y` / `yyyy`: Specifies the full 4-digit year;
         * - `yy`: Specifies the 2-digit year;
         * - `M`: Specifies the month number;
         * - `MM`: Specifies the month number with a minimum of 2 digits;
         * - `MMM`: Specifies the month as an abbreviated string which should be supplied through the `monthFormattedAbbreviated`
         * parameter (e.g. `Jan`);
         * - `MMMM`: Specifies the month as a full string which should be supplied through the `monthFormattedWide`
         * parameter (e.g. `January`);
         * - `MMMMM`: Specifies the month as a narrow string which should be supplied through the `monthFormattedNarrow`
         * parameter (e.g. `J`);
         * - `LLL`: Specifies the month as an abbreviated nominative string which should be supplied through
         * the `monthNominativeAbbreviated` parameter (e.g. `Jan`);
         * - `LLLL`: Specifies the month as a full nominative string which should be supplied through
         * the `monthNominativeWide` parameter (e.g. `January`);
         * - `LLLLL`: Specifies the month as a narrow nominative string which should be supplied through
         * the `monthNominativeNarrow` parameter (e.g. `J`);
         * - `d`: Specifies the day of the month;
         * - `dd`: Specifies the day of the month with a minimum of 2 digits;
         * - `E`: Specifies the day of the week;
         * - `EE`: Specifies the day of the week as a string which should be supplied through the `weekdaysFormattedShort`
         * parameter (e.g. `Su`);
         * - `EEE`: Specifies the day of the week as an abbreviated string which should be supplied through
         * the `weekdaysFormattedAbbreviated` parameter (e.g. `Sun`);
         * - `EEEE`: Specifies the day of the week as a full string which should be supplied through the `weekdaysFormattedWide`
         * parameter (e.g. `Sunday`);
         * - `EEEEE`: Specifies the day of the week as a narrow string which should be supplied through the `weekdaysFormattedNarrow`
         * parameter (e.g. `S`);
         * - `cc`: Specifies the day of the week as a nominative string which should be supplied through the `weekdaysNominativeShort`
         * parameter (e.g. `Su`);
         * - `ccc`: Specifies the day of the week as an abbreviated nominative string which should be supplied through
         * the `weekdaysNominativeAbbreviated` parameter (e.g. `Sun`);
         * - `cccc`: Specifies the day of the week as a full nominative string which should be supplied through
         * the `weekdaysNominativeWide` parameter (e.g. `Sunday`);
         * - `ccccc`: Specifies the day of the week as a narrow nominative string which should be supplied through
         * the `weekdaysNominativeNarrow` parameter (e.g. `S`);
         * - `H`: Specifies the number of hours in 24-hour format;
         * - `HH`: Specifies the number of hours in 24-hour format  with a minimum of 2 digits;
         * - `h`: Specifies the number of hours in 12-hour format;
         * - `hh`: Specifies the number of hours in 12-hour format  with a minimum of 2 digits;
         * - `a`: Specifies the `AM` or `PM` hour suffix;
         * - `m`: Specifies the number of minutes;
         * - `mm`: Specifies the number of minutes with a minimum of 2 digits;
         * - `s`: Specifies the number of seconds;
         * - `ss`: Specifies the number of seconds with a minimum of 2 digits;
         * - `S`: Specifies the number of milliseconds;
         * - `SSS`: Specifies the number of milliseconds with a minimum of 3 digits;
         * @param nTime Specifies the date/time to format. If this value is not specified `Now` will be used.
         * @param bUTC Specifies if the UTC time needs to be used.
         * @param pLocales Contains the date/time locales.
         * @return Returns the formatted date string.
         * @see http://cldr.unicode.org/translation/date-time
         */
        static format(
            sFormat?: string,
            nTime?: number,
            bUTC?: boolean,
            pLocales?: {
                /** Specifies an array with 12 formatted abbreviated month labels, starting with january. */
                monthsFormattedAbbreviated?: string[];
                /** Specifies an array with 12 formatted narrow month labels, starting with january. */
                monthsFormattedNarrow?: string[];
                /** Specifies an array with 12 formatted wide month labels, starting with january. */
                monthsFormattedWide?: string[];
                /** Specifies an array with 12 nominative abbreviated month labels, starting with january. */
                monthsNominativeAbbreviated?: string[];
                /** Specifies an array with 12 nominative narrow month labels, starting with january. */
                monthsNominativeNarrow?: string[];
                /** Specifies an array with 12 nominative wide month labels, starting with january. */
                monthsNominativeWide?: string[];
                /** Specifies an array with 7 formatted abbreviated day labels, starting with sunday. */
                weekdaysFormattedAbbreviated?: string[];
                /** Specifies an array with 7 formatted narrow day labels, starting with sunday. */
                weekdaysFormattedNarrow?: string[];
                /** Specifies an array with 7 formatted short day labels, starting with sunday. */
                weekdaysFormattedShort?: string[];
                /** Specifies an array with 7 formatted wide day labels, starting with sunday. */
                weekdaysFormattedWide?: string[];
                /** Specifies an array with 7 nominative abbreviated day labels, starting with sunday. */
                weekdaysNominativeAbbreviated?: string[];
                /** Specifies an array with 7 nominative narrow day labels, starting with sunday. */
                weekdaysNominativeNarrow?: string[];
                /** Specifies an array with 7 nominative short day labels, starting with sunday. */
                weekdaysNominativeShort?: string[];
                /** Specifies an array with 7 nominative wide day labels, starting with sunday. */
                weekdaysNominativeWide?: string[];
                /** Specifies the `AM` suffix. */
                AM?: string;
                /** Specifies the `PM` suffix. */
                PM?: string;
            }
        ): string;
        /**
         * Formats an elapsed time as a string. Uses the CLDR date/time formatting style.
         * @param sFormat Specifies the format to use. The following variables are available:
         * - `H`: Specifies the number of hours;
         * - `HH`: Specifies the number of hours with a minimum of 2 digits;
         * - `m`: Specifies the number of minutes;
         * - `mm`: Specifies the number of minutes with a minimum of 2 digits;
         * - `s`: Specifies the number of seconds;
         * - `ss`: Specifies the number of seconds with a minimum of 2 digits;
         * - `S`: Specifies the number of milliseconds;
         * - `SSS`: Specifies the number of milliseconds with a minimum of 3 digits;
         * - `u`: Specifies the number of microseconds;
         * - `uuu`: Specifies the number of microseconds with a minimum of 3 digits;
         * @param nElapsed Specifies the elapsed time. If this value is not specified, the current `timerElapsed` value will be used.
         * @param sAM Specifies the `AM` suffix.
         * @param sPM Specifies the `PM` suffix.
         * @return Returns the formatted elapsed time string.
         * @see http://cldr.unicode.org/translation/date-time
         */
        static formatElapsed(
            sFormat?: string,
            nElapsed?: number,
            sAM?: string,
            sPM?: string
        ): string;
        /** Returns a preformatted log string. */
        static readonly log: string;
        /** Returns a preformatted time log string. */
        static readonly logTime: string;
        /**
         * Constructs a new date object.
         * @param Specifies the date/time.
         */
        constructor(pDate?: DateTime | Date | number);
        /** Retrieves the date object. */
        readonly date: Date;
        /** Sets the unix time stamp. */
        time: number;
    }
}

declare module "@marblecore/functional/lib/flags" {
    export class Flags {
        /** Creates a new flag identifier. */
        readonly flag: number;
        /**
         * Converts the supplied boolean values to a flag identifier.
         * @param pFlags Array with boolean attributes.
         * @return Returns the flag number.
         */
        static convert(...pFlags: boolean[]): number;
        /**
         * Checks if the supplied flags value has the specified flag index enabled.
         * @param nFlags Specifies the flags value.
         * @param nIndex Specifies the flag index (the first flag is `1`).
         * @return Returns `true` if the flag is enabled.
         */
        static has(nFlags: number, nIndex: number): boolean;
    }
}

declare module "@marblecore/functional/lib/enumerator" {
    export class Enumerator {
        /** Retrieves the next global enumerator. */
        static readonly n: number;
        /** Sets the enumerator offset. */
        n: number;
        /** Recycles the global enumerator. */
        static recycle(): void;
        /** Recycles the instance enumerator. */
        recycle(): void;
    }
}

declare module "@marblecore/functional/lib/limiter" {
    import { TArguments } from "@marblecore/functional/lib/types";
    export type TType = "throttle" | "throttle-delay" | "debounce";
    export type TCallback = (...pArguments: TArguments) => void;
    export class Limiter {
        /**
         * Creates a new limiter.
         * @param sType Specifies the limiter type.
         * @param fnCallback Specifies the callback function.
         * @param nTimeout Specifies the timeout. Can be a number that specifies
         * the number of milliseconds or can be one of the following values:
         * - `tick`: Use a single tick;
         * - `frame`: Use a single animation frame;
         * - `double-frame`: Use a double animation frame;
         * - `promise`: Debounce using promise;
         * - `tick`: Debounce on next tick.
         */
        constructor(
            sType: TType,
            fnCallback: TCallback,
            nTimeout: number | "tick" | "promise" | "frame" | "double-frame"
        );
        /**
         * Invokes the callback with the supplied arguments.
         * @param pArguments Specifies the arguments to pass to the callback.
         */
        invoke(...pArguments: TArguments): void;
        /** Cancels any pending callbacks. */
        cancel(): void;
        /** Hold any pending callbacks. */
        hold(): void;
        /**
         * Release any pending callbacks.
         * @param bImmediately Specifies if the release should be instanyly.
         */
        release(bImmediately?: boolean): void;
    }
    export class Throttle extends Limiter {
        /**
         * Creates a new throttle.
         * @param fnCallback Specifies the callback function.
         * @param nTimeout Specifies the throttle timeout. Can be a number that
         * specifies the number of milliseconds or can be one of the following values:
         * - `frame`: Use a single animation frame (default);
         * - `double-frame`: Use a double animation frame;
         * - `promise`: Debounce using promise;
         * - `tick`: Debounce on next tick.
         * @param bDelay Specifies if the first call should be delayed until the
         * timeout. If the delay is disabled (`false`), all calls until the next
         * timeout are ignored.
         */
        constructor(
            fnCallback: TCallback,
            nTimeout?: number | "tick" | "promise" | "frame" | "double-frame",
            bDelay?: boolean
        );
    }
    export class Debounce extends Limiter {
        /**
         * Creates a new debouncer.
         * @param fnCallback Specifies the callback function.
         * @param nTimeout Specifies the debounce timeout. Can be a number that
         * specifies the number of milliseconds or can be one of the following values:
         * - `frame`: Use a single animation frame (default);
         * - `double-frame`: Use a double animation frame;
         * - `promise`: Debounce using promise;
         * - `tick`: Debounce on next tick.
         */
        constructor(
            fnCallback: TCallback,
            nTimeout?: number | "tick" | "promise" | "frame" | "double-frame"
        );
    }
}

declare module "@marblecore/functional/lib/stack" {
    export class Stack {
        /** Adds a call to the buffer. */
        add(fnCall: () => void): void;
        /** Executes all calls in the stack. */
        flush(): void;
        /** Clears the stack. */
        clear(): void;
        /** Flushes and clears the stack. */
        flushAndClear(): void;
    }
}
